import { GeolocationMapMarker } from "../../../js/GeolocationMapMarker.js";
import { GeolocationCoordinates } from "../../../js/GeolocationCoordinates.js";

/**
 * @property {BMap.Marker} baiduMarker
 * @property {Baidu} map
 */
export class BaiduMapMarker extends GeolocationMapMarker {
  constructor(coordinates, settings = {}, map = null) {
    super(coordinates, settings, map);

    let baiduMarkerSettings = {
      title: this.settings.title,
    };

    if (this.settings.icon) {
      baiduMarkerSettings.icon = new BMap.Icon(this.settings.icon, new BMap.Size(300, 157));
    }

    this.baiduMarker = new BMap.Marker(new BMap.Point(coordinates.lng, coordinates.lat), baiduMarkerSettings);

    if (this.settings.label) {
      this.baiduMarker.setLabel(new BMap.Label(this.settings.label));
    }

    this.baiduMarker.addEventListener("click", () => {
      this.click();
    });

    if (this.settings.draggable) {
      this.baiduMarker.enableDragging();
      this.baiduMarker.addEventListener("dragend", (e) => {
        this.update(new GeolocationCoordinates(Number(e.point.lng), Number(e.point.lat)));
      });
    }
  }

  update(newCoordinates, settings) {
    super.update(newCoordinates, settings);

    if (newCoordinates) {
      if (!newCoordinates.equals(this.baiduMarker.getPosition().lat, this.baiduMarker.getPosition().lng)) {
        this.baiduMarker.setPosition(new BMap.Point(newCoordinates.lng, newCoordinates.lat));
      }
    }

    if (this.settings.title) {
      this.baiduMarker.setTitle(this.settings.title);
    }
    if (this.settings.label) {
      this.baiduMarker.setLabel(new BMap.Label(this.settings.label));
    }
    if (this.settings.icon) {
      this.baiduMarker.setIcon(new BMap.Icon(this.settings.icon, new BMap.Size(300, 157)));
    }
  }

  remove() {
    super.remove();

    this.map.baiduMap.removeOverlay(this.baiduMarker);
  }
}
